@@ -1,4 +1,4 @@
-/* HTGET (C) J Whitham 1998
+/* HTGET (C) J Whitham 1998-99
 ** based on snarf, the Simple Non-interactive All-purpose Resource Fetcher
 ** snarf is copyright (C) 1995, 1996 Zachary Beane
 **
@@ -20,20 +20,21 @@
 ** jwhitham@globalnet.co.uk
 */
 
-/* htget version 0.92 */
+/* htget version 0.93 */
 
-#include <stdio.h>
-#include <string.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/stat.h>
-#include <netinet/in.h>
-#include <unistd.h>
+#include <ctype.h>
+#include <errno.h>
 #include <fcntl.h>
-#include <string.h>
 #include <netdb.h>
+#include <netinet/in.h>
+#include <signal.h>
 #include <stdio.h>
-#include <time.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/socket.h>
+#include <sys/time.h>
+#include <sys/types.h>
+#include <unistd.h>
 
 #define MAXLEN 		256
 #define BIG_BUFFER_SIZE 4096
@@ -89,15 +90,17 @@ int ReadLine ( int Socket , char * ReceiveBuffer ) ;
 char * ToBase64(unsigned char *bin, int len ) ;
 int ProcessURL ( char * TheURL , char * Hostname , char * Filename , char * ActualFilename , unsigned * Port ) ;
 int GetHeaderForRequest ( unsigned socket , char * request , char * Header ) ;
-void PrintReport ( long ArrivedSize , long RangeSize , 
-	time_t CurrentTime , time_t StartTime , time_t TimeoutTime ) ;
-			
+int GetHeaderData(const char *Header, const char *FieldName, char *Record);
+int HTGet(char *RetrieveURL, int AllowResume);
+void PrintReport(long ArrivedSize, long RangeSize,
+                 const struct timeval *StartTime, int TimeoutTime);
+void tick(int signal);
 			
 
 int main ( int argc , char * argv [] )
 {
 	char		Temp [ 130 ] ;
-	int			I , J , URLs , Ok , CurrentURL , Success ;
+	int			I , URLs , Ok , CurrentURL , Success ;
 	FILE		* ResourceFile ;
 	char		* Home ;
 	char		RetrieveURL [ MAX_URLS ][ 128 ] ;
@@ -157,7 +160,7 @@ int main ( int argc , char * argv [] )
 	ShowPercentage = 1 ;
 	ShowBytesPerSecond = 1 ;
 	strcpy ( DownloadsDir , "./" ) ;
-	strcpy ( UserAgent , "linux htget 0.92" ) ;
+	strcpy ( UserAgent , "linux htget 0.93" ) ;
 	ShowBanner = 1 ;
 
 	if ( ResourceFile != NULL )
@@ -166,7 +169,7 @@ int main ( int argc , char * argv [] )
 		{
 			fgets ( Temp , 128 , ResourceFile ) ;
 			/* Remove terminating newline/junk */
-			for ( I = 0 ; I < strlen ( Temp ) ; I ++ )
+			for ( I = 0 ; I < (signed int) strlen ( Temp ) ; I ++ )
 			{
 				if ( ! isprint ( Temp [ I ] ))
 				{
@@ -242,7 +245,7 @@ int main ( int argc , char * argv [] )
 
 	if ( ShowBanner )
 	{
-		fprintf ( stderr , "HTGET (c) J Whitham 1998  <jwhitham@globalnet.co.uk> version 0.92\n" 
+		fprintf ( stderr , "HTGET (c) J Whitham 1998-99  <jwhitham@globalnet.co.uk> version 0.93\n" 
 			"See README. This program comes with NO WARRANTY of any kind.\n\n" ) ;
 	}
 
@@ -376,7 +379,7 @@ int main ( int argc , char * argv [] )
 						{
 							CurrentURL ++ ;
 						} else {
-							fprintf ( stderr , "\nRedial: delaying for %d seconds before retry.." , Redial ) ;
+							fprintf ( stderr , "\nRedial: delaying for %ld seconds before retry.." , Redial ) ;
 							sleep ( Redial ) ; 
 							fprintf ( stderr , "\n" ) ;
 						}
@@ -391,7 +394,7 @@ int main ( int argc , char * argv [] )
 			case ERROR_TIMEOUT :	/* just try again */
 						if ( Redial > 0 )
 						{
-							fprintf ( stderr , "\nRedial: delaying for %d seconds before retry.." , Redial ) ;
+							fprintf ( stderr , "\nRedial: delaying for %ld seconds before retry.." , Redial ) ;
 							sleep ( Redial ) ; 
 							fprintf ( stderr , "\n" ) ;
 						}
@@ -409,30 +412,30 @@ int HTGet ( char * RetrieveURL , int AllowResume )
 	char		Header [ BIG_BUFFER_SIZE ] ;
 	char		Request [ BIG_BUFFER_SIZE ] ;
 	/* tcp/ip */
-	unsigned	socket ;
+	unsigned int	socket ;
 	/* URL */
 	int		I , Port ;
-	char		ch ;
 	char		Hostname [ MAXLEN ] ;
 	char		Filename [ MAXLEN ] ;
 	char		ActualFilename [ MAXLEN ] ;
-	char		Parameter [ MAXLEN ] ;
 	int		Resuming = 0 ;
 	/* output */
 	FILE		* File ;
-	long		ActualSize , ArrivedSize ;
+	long		ActualSize;
  	char 		recv_buf [ BIG_BUFFER_SIZE ] ;
-	int		Percentage ;
 	long		CurrentSize ;
-	long		RangeSize = 0 , RangeStart = 0 ;
+	long		RangeStart = 0 ;
 	/* authenticate */
 	char		UserPass [ MAXLEN ] ;
-	struct stat	ConnectionStatistics ;
 	char		* base64str ;
-	/* timing */
-	time_t		StartTime , CurrentTime , TimeoutTime ;
-	time_t		RedundantReportsTimer ;
 
+	/* import status globals */
+  extern int ArrivedSize;
+  extern int RangeSize;
+  extern struct timeval StartTime;
+  /* extern int ticks; */
+
+  RangeSize = 0;
 
 	strcpy ( UserPass , USER_PASS ) ;
 	
@@ -447,7 +450,7 @@ int HTGet ( char * RetrieveURL , int AllowResume )
 	{
 		if ( DownloadsDir [ I ] == '/' )
 		{
-			DownloadsDir [ I ] == '\0' ;
+			DownloadsDir [ I ] = '\0' ;
 		} else {
 			break ;
 		}
@@ -630,71 +633,76 @@ int HTGet ( char * RetrieveURL , int AllowResume )
 			gizmo to try again from 0 bytes */
 	}
 	fseek ( File , RangeStart , SEEK_SET ) ;
- 
  	
-	fprintf ( stderr , "\nReceiving HTTP data.." ) ;
-	I = read ( socket , recv_buf , sizeof ( recv_buf ) ) ;
-	fcntl ( socket , F_SETFL , O_NONBLOCK ) ;
-	
-	ArrivedSize = 0 ;
-	fprintf ( stderr , "..\nTotal size: %ld bytes\n" , ActualSize ) ;
-	StartTime = time ( NULL ) ;
-	TimeoutTime = StartTime + Timeout ;
-	while ( I > 0 )
-	{	
-		if ( fwrite ( recv_buf , I , 1 , File ) != 1 )
-		{
-			fprintf ( stderr , "\nError writing to file %s\n" , ActualFilename ) ;
-			fclose ( File ) ;
-			return ( ERROR_WRITE ) ;
-		}
-		ArrivedSize += I ;
-		CurrentTime = time ( NULL ) ;
-		if ( MinRate < 1 )
-		{
-			TimeoutTime = CurrentTime + Timeout ;
-		} else {
-			/* BytesPerSecond = ArrivedSize / ( CurrentTime - StartTime ) ; */
-			if (( ArrivedSize > 0 ) 
-			&& ( CurrentTime > StartTime ))
-			{
-				if (( ArrivedSize / ( CurrentTime - StartTime )) < MinRate )
-				{
-					/* just not good enough */
-				} else {
-					TimeoutTime = CurrentTime + Timeout ;
-				}
-			} else {
-				TimeoutTime = CurrentTime + Timeout ;
-			}
-		}
-		I = -1 ;
-		RedundantReportsTimer = 0 ;
-		while ( I < 0 )
-		{
-			I = read ( socket , recv_buf , sizeof ( recv_buf ) ) ;
-			if ( RedundantReportsTimer != CurrentTime )
-			{
-				PrintReport ( ArrivedSize , RangeSize , CurrentTime , StartTime , TimeoutTime ) ;
-				RedundantReportsTimer = CurrentTime ;
-			}
-			CurrentTime = time ( NULL ) ;
-			if ( Timeout < 1 )
-			{
-				TimeoutTime = CurrentTime + 120 ;
-			}
-			if ( CurrentTime > TimeoutTime )
-			{
-				fclose ( File ) ;
-				return ( ERROR_TIMEOUT ) ;
-			}
-		}
-	}
-	fclose ( File ) ;
-	fprintf ( stderr , "\nFinished downloading.\n" ) ;
-	return ( DOWNLOAD_OK ) ;
-}
+  fprintf(stderr, "\nReceiving HTTP data..");
+  fprintf(stderr, "..\nTotal size: %ld bytes\n", ActualSize);
 
+  gettimeofday(&StartTime, NULL);
+
+  {
+  struct itimerval timer;
+  struct sigaction action;
+  extern int ticks;
+
+  /* setup a signal handler */
+  memset(&action, 0, sizeof(action));
+  action.sa_handler = tick;
+  sigaction(SIGALRM, &action, NULL);
+
+  /* start the timer ticking */
+  timer.it_interval.tv_sec = 1;
+  timer.it_interval.tv_usec = 0;
+  timer.it_value.tv_sec = 1;
+  timer.it_value.tv_usec = 0;
+  setitimer(ITIMER_REAL, &timer, NULL);
+
+  ticks = Timeout;
+  ArrivedSize = 0;
+
+  while ((I = read(socket, recv_buf, sizeof(recv_buf))))
+  {
+    if (I > 0)
+    {
+      ticks = Timeout;
+
+      if (fwrite(recv_buf, I, 1, File) != 1)
+      {
+        setitimer(ITIMER_REAL, NULL, NULL);
+        fprintf(stderr, "\nError writing to file %s\n", ActualFilename);
+        fclose(File);
+        return (ERROR_WRITE);
+      }
+
+      if ((ArrivedSize += I) == RangeSize)
+      {
+        setitimer(ITIMER_REAL, NULL, NULL);
+        fclose(File);
+        fprintf(stderr, "\nFinished downloading.\n");
+        return (DOWNLOAD_OK);
+      }
+    }
+    else if (errno != EINTR)
+    {
+      setitimer(ITIMER_REAL, NULL, NULL);
+      fprintf(stderr, "\nError reading from web server\n");
+      fclose(File);
+      return (ERROR_HTTP);
+    }
+
+    if (ticks <= 0)
+    {
+      setitimer(ITIMER_REAL, NULL, NULL);
+      fclose(File);
+      return (ERROR_TIMEOUT);
+    }
+  }
+  }
+  /* stop the timer */
+  setitimer(ITIMER_REAL, NULL, NULL);
+  fclose(File);
+  fprintf(stderr, "\nFinished downloading.\n");
+  return (DOWNLOAD_OK);
+}
 
 
 /* TCPConnect
@@ -843,7 +851,7 @@ int ProcessURL ( char * TheURL , char * Hostname , char * Filename , char * Actu
 		return ( 0 ) ;
 	}
 	/* scan for first /. */
-	for ( I = 0 ; I < strlen ( NormalURL ) ; I ++ )
+	for ( I = 0 ; I < (signed int) strlen ( NormalURL ) ; I ++ )
 	{
 		if ( NormalURL [ I ] == '/' )
 		{
@@ -908,7 +916,7 @@ int GetHeaderForRequest ( unsigned socket , char * request , char * Header )
 	char		line [ MAXLEN ] ;
 	char		status [ MAXLEN ] ;
 	char		token [ MAXLEN ] ;
-	int 		in_header;
+	int 		in_header = 0;
 	char		ch ;
 
 	Header [ 0 ] = '\0' ;
@@ -954,7 +962,7 @@ int GetHeaderForRequest ( unsigned socket , char * request , char * Header )
 	strcpy ( status , line ) ;
 
 	/* scan line for "HTTP" to indicate this is the header */
-	for ( I = 0 ; I < ( strlen ( line ) - strlen ( HEADER_INDICATOR ) ) ; I ++ )
+	for ( I = 0 ; I < (signed int) ( strlen ( line ) - strlen ( HEADER_INDICATOR ) ) ; I ++ )
 	{
 		if ( strncmp ( & line [ I ] , HEADER_INDICATOR , strlen ( HEADER_INDICATOR ) ) == 0 )
 		{
@@ -998,7 +1006,7 @@ int GetHeaderForRequest ( unsigned socket , char * request , char * Header )
 }
 
 
-int NoCaseStrNCmp ( char * String1 , char * String2 , int Length )
+int NoCaseStrNCmp ( const char * String1 , const char * String2 , int Length )
 {
 	int	I ;
 
@@ -1012,19 +1020,19 @@ int NoCaseStrNCmp ( char * String1 , char * String2 , int Length )
 	return ( 1 ) ;
 }
 
-int GetHeaderData ( char * Header , char FieldName [] , char * Record )
+int GetHeaderData(const char *Header, const char *FieldName, char *Record)
 {
 	int	I , J ;
 
-	for ( I = 0 ; I < strlen ( Header ) ; I ++ )
+	for ( I = 0 ; I < (signed int) strlen ( Header ) ; I ++ )
 	{
 		if ( NoCaseStrNCmp ( & Header [ I ] , FieldName , strlen ( FieldName ) ) )
 		{
 			I += strlen ( FieldName ) ;
-			if ( ( I + 2 ) < strlen ( Header ))
+			if ( ( I + 2 ) < (signed int) strlen ( Header ))
 			{
 				strcpy ( Record , & Header [ I ] ) ;
-				for ( J = 0 ; J < strlen ( Record ) ; J ++ )
+				for ( J = 0 ; J < (signed int) strlen ( Record ) ; J ++ )
 				{
 					if ( Record [ J ] == '\n' )
 					{
@@ -1042,41 +1050,39 @@ int GetHeaderData ( char * Header , char FieldName [] , char * Record )
 }
 
 
-void PrintReport ( long ArrivedSize , long RangeSize , 
-	time_t CurrentTime , time_t StartTime , time_t TimeoutTime )
+void PrintReport(long ArrivedSize, long RangeSize,
+                 const struct timeval *StartTime, int TimeoutTime)
 {
-	long		Percentage ;
-	long		BytesPerSecond ;
-	time_t		TimeLeft ;
-	long		BytesLeft ;
+  float now;
+  float start;
+  float BytesPerSecond;
+  int TimeLeft;
+
+  struct timeval CurrentTime;
+  gettimeofday(&CurrentTime, NULL);
 
 	fprintf ( stderr , "\r\033[K\r.." ) ;
 	if ( ShowBytesRemaining )
-	{
-		if ( RangeSize > 0 )
-		{
-			fprintf ( stderr , " %ld/%ld" , ArrivedSize , RangeSize ) ;
-		} else {
-			fprintf ( stderr , " %ld/unknown" , ArrivedSize ) ;
-		}
-	}
-	if (( ShowPercentage ) && ( RangeSize > 0 ) )
-	{
-		Percentage = ( ArrivedSize * 100 ) / RangeSize ;
-		fprintf ( stderr , " %d%%" , Percentage ) ;
-	}
-	if ( ( CurrentTime - StartTime ) > 0 )
-	{
-		BytesPerSecond = ArrivedSize / ( CurrentTime - StartTime ) ;
-		if ( ShowBytesPerSecond )
-		{
-			fprintf ( stderr , " %ld bytes/sec" , BytesPerSecond ) ;
-		}
+  {
+    if (RangeSize > 0)
+      fprintf(stderr, " %ld/%ld", ArrivedSize, RangeSize);
+    else
+      fprintf(stderr, " %ld/unknown", ArrivedSize);
+  }
+  if ((ShowPercentage) && (RangeSize > 0))
+    fprintf(stderr, " %.2f%%", (ArrivedSize * 100) / (float) RangeSize);
+
+  start = StartTime->tv_sec + StartTime->tv_usec / 1000000;
+  now = CurrentTime.tv_sec + CurrentTime.tv_usec / 1000000;
+  if ((now - start) > 0)
+  {
+    BytesPerSecond = ArrivedSize / (now - start);
+    if (ShowBytesPerSecond)
+      fprintf(stderr, " %.2f bytes/sec", BytesPerSecond);
 		if (( RangeSize > 0 ) && ShowTimeRemaining )
 		{
 			/* time left may be predicted. */
-			BytesLeft = RangeSize - ArrivedSize ;
-			TimeLeft = BytesLeft / BytesPerSecond ;
+      TimeLeft = (RangeSize - ArrivedSize) / BytesPerSecond;
 			if ( TimeLeft > 3599 )
 			{
 				/* hours and minutes */
@@ -1089,9 +1095,27 @@ void PrintReport ( long ArrivedSize , long RangeSize ,
 			}
 		}
 	}
-	if (( CurrentTime < TimeoutTime )
-	&& ( CurrentTime > ( TimeoutTime - 30 )))
-	{
-		fprintf ( stderr , " Timeout in %lds" , ( TimeoutTime - CurrentTime ) ) ;
-	}
+  if (TimeoutTime < (Timeout - 1))
+    fprintf(stderr, " Timeout in %ds", TimeoutTime);
 }
+
+/* this is the alarm signal handler... which decrements the timeout timer */
+void tick(int signal)
+{
+  extern int ArrivedSize;
+  extern int RangeSize;
+  extern struct timeval StartTime;
+  extern int ticks;
+
+  signal = signal;
+  ticks--;
+
+  /* print out the report for each alarm */
+  PrintReport(ArrivedSize, RangeSize, &StartTime, ticks);
+}
+
+/* ick globals */
+int ArrivedSize;
+int RangeSize;
+struct timeval StartTime;
+int ticks;
